@@ -1,11 +1,10 @@
-from threading import Lock
+from threading import Lock, Event
 from datetime import timedelta
 from typing import Union, List, Any, Callable
 from time import sleep
 from torch.distributed import rpc
 from machin.parallel.exception import ExceptionWithTraceback
 
-import os
 import enum
 import torch as t
 import torch.distributed as dist
@@ -189,10 +188,6 @@ def get_world():  # pragma: no cover
     return WORLD
 
 
-def _get_rpc_group(group_name):  # pragma: no cover
-    return WORLD.groups.get(group_name, None)
-
-
 def _is_group_ready(group_name):  # pragma: no cover
     return WORLD.group_create_signals.get(group_name, None) is False
 
@@ -332,6 +327,9 @@ class World:
             raise RuntimeError("Group {} already exists!".format(group_name))
         group = RpcGroup(group_name, members)
 
+        # set the group
+        self.groups[group_name] = group
+
         # temporarily set a signal
         self.group_create_signals[group_name] = False
         # wait for other members to enter
@@ -357,37 +355,8 @@ class World:
         else:
             while self.group_create_signals[group_name] is not True:
                 sleep(0.1)
-
-        # set the group
-        self.groups[group_name] = group
         return group
 
-    def get_rpc_group(self, group_name: str, target: str = None):
-        """
-        Get group with name ``group_name``, supports group not created
-        on this process. You should only use it if you really needs to
-        access a value/service only exposed in the target group.
-
-        Warning:
-            This API is obsolete.
-
-        Args:
-            group_name: Group name.
-            target: Target process used to query for the group info
-                if it is not created locally, by default it is set
-                to the local process.
-
-        Returns:
-            Target group, ``None`` if not found
-        """
-
-        if target is None:
-            return self.groups.get(group_name, None)
-        else:
-            return rpc.rpc_sync(target,
-                                _get_rpc_group,
-                                args=(group_name,))
-
     def get_members(self):
         """
         Returns:
@@ -564,12 +533,19 @@ class CollectiveGroup:
 # add the heartbeat mechanism to the lut_manager, to increase robustness
 
 class RpcGroup:
-    def __init__(self, group_name, group_members):
+    def __init__(self, group_name, group_members, first_create=True):
         self.group_name = group_name
         self.group_members = group_members
         self.group_value_lut = {}
         self.group_service_lut = {}
         self.destroyed = False
+        self._barrier_event = Event()
+        self._barrier_status = False
+        if first_create and self.is_member(get_cur_name()):
+            self.register("_rpc_entered_barrier_{}".format(get_cur_name()),
+                          self._rpc_entered_barrier)
+            self.register("_rpc_exit_barrier_{}".format(get_cur_name()),
+                          self._rpc_exit_barrier)
 
     @_copy_doc(rpc.rpc_sync)
     def rpc_sync(self, to: str, func: Callable,
@@ -811,20 +787,30 @@ class RpcGroup:
             kwargs = {}
         return self._rpc_service_call(rpc.remote, key, args, kwargs)
 
+    @_check_executor
     def barrier(self):
         """
         Synchronize all members in the group, until all members have entered
         a ``barrier()`` function.
+
+        Not thread-safe.
         """
-        self.pair("__enter_barrier_{}".format(get_cur_name()),
-                  True)
-        while True:
-            paired = [self.is_paired("__enter_barrier_{}".format(m))
-                      for m in self.group_members]
-            if all(paired):
-                break
-            else:
-                sleep(0.2)
+
+        self._barrier_status = True
+        if get_cur_name() == self.group_members[0]:
+            while True:
+                all_entered = all(
+                    self.registered_sync("_rpc_entered_barrier_{}".format(m))
+                    for m in self.group_members
+                )
+                if not all_entered:
+                    sleep(0.2)
+                else:
+                    break
+            for m in self.group_members:
+                self.registered_sync("_rpc_exit_barrier_{}".format(m))
+        else:
+            self._barrier_event.wait()
 
     @_check_executor
     def destroy(self):
@@ -893,6 +879,14 @@ class RpcGroup:
         return rpc_method(holder, _rpc_call_service,
                           args=(self.group_name, key, args, kwargs))
 
+    def _rpc_entered_barrier(self):
+        return self._barrier_status
+
+    def _rpc_exit_barrier(self):
+        self._barrier_status = False
+        self._barrier_event.set()
+        self._barrier_event.clear()
+
     def __reduce__(self):  # pragma: no cover
         # returns a complete description of group
-        return RpcGroup, (self.group_name, self.group_members)
+        return RpcGroup, (self.group_name, self.group_members, False)
